let (linkstatus = {})
{
  if ('linkstatus_252c06c0_17cb_42e3_b034_57cbbf8b4bdb' in window)
    throw new Error('linkstatus: The script is initialized more than once.  Something is wrong.');

  // Expose the object just in case someone ever wants to customize this extension.
  window.linkstatus_252c06c0_17cb_42e3_b034_57cbbf8b4bdb = linkstatus;

  linkstatus.prefs = null;
  linkstatus.prefShowLastVisited = true;
  linkstatus.prefEnableDirectSQL = true; // If false, we do not try to use nsPIPlacesDatabase
  linkstatus.prefVerboseLog = false;
  linkstatus.prefSlowQueriesThreshold = 30; // Report a query of last-visited date as slow if it takes at least this time in milliseconds

  linkstatus.overLinkVisited = false;
  linkstatus.overLinkBookmarked = false;
  linkstatus.statusHistoryState = null;
  linkstatus.lastTimeVisited = null;
  linkstatus.statusLastTimeVisited = null;

  linkstatus.startup = function() {
    this.initialized = true;
    this.strings = document.getElementById('linkstatus-strings');

    this.consolesvc = Components.classes['@mozilla.org/consoleservice;1']
        .getService(Components.interfaces.nsIConsoleService);

    Components.utils.import("resource://gre/modules/PluralForm.jsm");
    var pluralRule = Number(this.strings.getString('pluralRule'));
    [this.pluralGet, this.pluralNumForms] = PluralForm.makeGetter(pluralRule);

    this.prefs = Components.classes['@mozilla.org/preferences-service;1']
        .getService(Components.interfaces.nsIPrefService)
        .getBranch('linkstatus.');
    this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this.prefs.addObserver('', this, false);

    this.prefShowLastVisited = this.prefs.getBoolPref('showLastVisited');
    this.prefEnableDirectSQL = this.prefs.getBoolPref('enableDirectSQL');
    this.prefVerboseLog = this.prefs.getBoolPref('verboseLog');
    this.prefSlowQueriesThreshold = this.prefs.getIntPref('slowQueriesThreshold');

    // Accessing the Places database directly for performance
    // See https://developer.mozilla.org/en/Updating_extensions_for_Firefox_3.5#Accessing_the_Places_database .
    // The nsPIPlacesDatabase interface does not exist in Firefox 3.0.x.
    // We do not bother to implement the older way to access Places database
    // because the Places database in Firefox 3.0.x uses an older schema (schema version 6)
    // and accessing it directly does not improve performance anyway.
    this.placesDB = null;
    this.stmtLastVisited = null;
    if (!('nsPIPlacesDatabase' in Components.interfaces))
    {
      if (this.prefVerboseLog)
        this.consolesvc.logStringMessage('linkstatus.startup: Fast lookup is disabled - Interface nsPIPlacesDatabase is unknown');
    }
    else
    {
      this.placesDB = Components.classes['@mozilla.org/browser/nav-history-service;1']
          .getService(Components.interfaces.nsPIPlacesDatabase)
          .DBConnection;
    }
    if (this.placesDB)
    {
      var dbSchema = this.placesDB.schemaVersion;
      // Places database with schema version < 9 cannot be used since moz_places table does not have last_visit_date field.
      // The difference between 9 and 10 is irrelevant to us.
      // Schema version == 9 for Firefox 3.5.x, 10 for Firefox 3.6a1.
      // We do not assume anything about future schema versions (> 10).
      if (dbSchema < 9 || dbSchema > 10)
      {
        if (this.prefVerboseLog)
          this.consolesvc.logStringMessage('linkstatus.startup: Fast lookup is disabled - Places DB has unknown schema version: ' + dbSchema);
        this.placesDB = null;
      }
    }
    if (this.placesDB)
    {
      try
      {
        this.stmtLastVisited = this.placesDB.createStatement(
          'SELECT last_visit_date ' +
          'FROM moz_places_temp ' +
          'WHERE ( url = :url ) ' +
          'UNION ALL ' +
          'SELECT last_visit_date ' +
          'FROM moz_places ' +
          'WHERE id NOT IN (SELECT id FROM moz_places_temp) AND ( url = :url ) ' +
          'LIMIT 1');
      }
      catch (e)
      {
        throw new Error('linkstatus.startup: createStatement failed: ' + this.placesDB.lastErrorString);
      }
      if (this.prefVerboseLog)
        this.consolesvc.logStringMessage('linkstatus.startup: Fast lookup is enabled');
    }

    this.setOverLinkOrig = XULBrowserWindow.setOverLink;
    XULBrowserWindow.setOverLink = function(link, b) {
      linkstatus.beforeSetOverLink(link);
      linkstatus.setOverLinkOrig.call(this, link, b);
    };
    this.updateStatusFieldOrig = XULBrowserWindow.updateStatusField;
    XULBrowserWindow.updateStatusField = function() {
      linkstatus.beforeUpdateStatusField();
      linkstatus.updateStatusFieldOrig.call(this);
    };
  };

  linkstatus.shutdown = function() {
    this.prefs.removeObserver('', this);
  };

  linkstatus.observe = function(subject, topic, data) {
    if (topic != "nsPref:changed")
      return;
    switch(data)
    {
      case 'showLastVisited':
        this.prefShowLastVisited = this.prefs.getBoolPref('showLastVisited');
        break;
      case 'enableDirectSQL':
        this.prefEnableDirectSQL = this.prefs.getBoolPref('enableDirectSQL');
        break;
      case 'verboseLog':
        this.prefVerboseLog = this.prefs.getBoolPref('verboseLog');
        break;
      case 'slowQueriesThreshold':
        this.prefSlowQueriesThreshold = this.prefs.getIntPref('slowQueriesThreshold');
        break;
    }
  };

  linkstatus.beforeSetOverLink = function(link) {
    this.overLinkVisited = false;
    this.overLinkBookmarked = false;
    this.lastTimeVisited = null;
    if (link)
    {
      var uri;
      var iosvc = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
      try
      {
        uri = iosvc.newURI(link, null, null);
      }
      catch (e if e.result == Components.results.NS_ERROR_MALFORMED_URI)
      {
        uri = null;
      }
      if (uri)
      {
        var times = null;
        if (this.prefVerboseLog)
          times = [Date.now()];
        var histsvc = Components.classes["@mozilla.org/browser/nav-history-service;1"].getService(Components.interfaces.nsIGlobalHistory2);
        if (histsvc.isVisited(uri))
        {
          if (times)
            times[1] = Date.now();
          this.overLinkVisited = true;
          if (linkstatus.prefShowLastVisited)
          {
            if (this.stmtLastVisited && this.prefEnableDirectSQL)
            {
              this.stmtLastVisited.params.url = uri.spec;
              if (times)
                times[2] = times[3] = Date.now();
              if (this.stmtLastVisited.executeStep())
              {
                if (times)
                  times[4] = Date.now();
                linkstatus.lastTimeVisited = '(' + linkstatus.beautifyDate(this.stmtLastVisited.row.last_visit_date / 1000) + ')';
                if (times)
                  times[5] = Date.now();
              }
              else
              {
                if (times)
                  times[4] = times[5] = Date.now();
              }
              this.stmtLastVisited.reset();
              if (times)
                times[6] = Date.now();
            }
            else
            {
              histsvc.QueryInterface(Components.interfaces.nsINavHistoryService);
              var query = histsvc.getNewQuery();
              query.uri = uri;
              var queryOptions = histsvc.getNewQueryOptions();
              queryOptions.includeHidden = true;
              queryOptions.maxResults = 1;
              if (times)
                times[2] = Date.now();
              var results = histsvc.executeQuery(query, queryOptions).root;
              if (times)
                times[3] = Date.now();
              results.containerOpen = true;
              if (times)
                times[4] = Date.now();
              if (results.hasChildren)
              {
                var result = results.getChild(0);
                linkstatus.lastTimeVisited = '(' + linkstatus.beautifyDate(result.time / 1000) + ')';
                if (times)
                  times[5] = Date.now();
              }
              else
              {
                if (times)
                  times[5] = times[4];
              }
              results.containerOpen = false;
              if (times)
                times[6] = Date.now();
            }
          }
          else
          {
            if (times)
              times[2] = times[3] = times[4] = times[5] = times[6] = times[1];
          }
        }
        else
        {
          if (times)
            times[1] = times[2] = times[3] = times[4] = times[5] = times[6] = Date.now();;
        }
        var bmsvc = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Components.interfaces.nsINavBookmarksService);
        this.overLinkBookmarked = bmsvc.isBookmarked(uri);
        if ("nsIBookmarksService" in Components.interfaces) {
          try {
            this.overLinkBookmarked = Components.classes["@mozilla.org/browser/bookmarks-service;1"]
                                                .getService(Components.interfaces.nsIBookmarksService)
                                                .isBookmarked(uri.spec);
          }
          catch(e) {
          }
        }
        if (times)
        {
          times[7] = Date.now();
          if (times[times.length - 1] - times[0] >= this.prefSlowQueriesThreshold)
          {
            var delta = [];
            for (var iii = 0; iii < times.length - 1; iii++)
              delta[iii] = times[iii + 1] - times[iii];
            this.consolesvc.logStringMessage('linkstatus: ' + (new Date(times[times.length - 1])).toString() + ': Possible performance problem: beforeSetOverLink took ' + (times[times.length - 1] - times[0]) + ' ms (' + delta.join(', ') + ')');
          }
        }
      }
    }
  };

  linkstatus.beforeUpdateStatusField = function() {
    var historyState = null;
    if (XULBrowserWindow.overLink)
    {
      if (this.overLinkBookmarked)
        historyState = 'bookmarked';
      else if (this.overLinkVisited)
        historyState = 'visited';
    }
    if (!historyState)
    {
      if (this.statusHistoryState)
      {
        XULBrowserWindow.statusTextField.removeAttribute('linkstatus-state');
        XULBrowserWindow.statusTextField.className = '';
        this.statusHistoryState = historyState;
      }
    }
    else
    {
      if (!this.statusHistoryState)
        XULBrowserWindow.statusTextField.className = 'statusbarpanel-text-linkstatus';
      if (this.statusHistoryState != historyState)
      {
        XULBrowserWindow.statusTextField.setAttribute('linkstatus-state', historyState);
        this.statusHistoryState = historyState;
      }
    }
    if (linkstatus.statusLastTimeVisited != linkstatus.lastTimeVisited)
    {
      if (linkstatus.lastTimeVisited)
        XULBrowserWindow.statusTextField.setAttribute('last-visited', linkstatus.lastTimeVisited);
      else
        XULBrowserWindow.statusTextField.removeAttribute('last-visited');
      linkstatus.statusLastTimeVisited = linkstatus.lastTimeVisited;
    }
  };

  linkstatus.beautifyDate = function(date) {
    var elapsed = new Date() - date;
    if (elapsed < 1000 * 60 * 60)
    {
      var minutes = Math.round(elapsed / (1000 * 60));
      var minutesAgo = this.strings.getString('minutesAgo');
      return this.pluralGet(minutes, minutesAgo).replace('#1', minutes);
    }
    else if (elapsed < 1000 * 60 * 60 * 24)
    {
      var hours = Math.round(elapsed / (1000 * 60 * 60));
      var hoursAgo = this.strings.getString('hoursAgo');
      return this.pluralGet(hours, hoursAgo).replace('#1', hours);
    }
    else if (elapsed < 1000 * 60 * 60 * 24 * 7.5)
    {
      var days = Math.round(elapsed / (1000 * 60 * 60 * 24));
      var daysAgo = this.strings.getString('daysAgo');
      return this.pluralGet(days, daysAgo).replace('#1', days);
    }
    else
    {
      // XXX: This returns the date in the browser's current locale,
      // which may not be the same as the extension's locale
      // (when the extension is not localized to the browser's locale).
      // However, this is difficult to fix because apparently
      // there is not a method to convert a given numerical date
      // to a string in the specified locale.
      return (new Date(date)).toLocaleDateString();
    }
  };

  // In the following statements,
  // "function(event) { linkstatus.startup(event); }" etc. cannot be replaced
  // by "linkstatus.startup" because it changes "this".
  window.addEventListener('load', function(event) { linkstatus.startup(event); }, false);
  window.addEventListener('unload', function(event) { linkstatus.shutdown(event); }, false);
}
